﻿namespace Sharing.DaoShu110
{
    /// <summary>
    /// A unified format for processing data returns
    /// </summary>
    public class ApiResult : IApiResult
    {
        /// <summary>
        /// Represents an empty <see cref="IApiResult"/>.
        /// </summary>
        public static readonly IApiResult Empty = new ApiResult
        {
            code = 1,
            msg = "请求成功",
        };

        /// <summary>
        /// Gets or sets the status code.
        /// </summary>
        /// <value>The status code.</value>
        public int code { get; set; }

        /// <summary>
        /// Gets or sets the message.
        /// </summary>
        /// <value>The message.</value>
        public string msg { get; set; }

        /// <summary>
        /// Creates a new instance of <see cref="IApiResult{TResult}"/> by the specified result.
        /// </summary>
        /// <typeparam name="TResult">The type of the result.</typeparam>
        /// <param name="result">The result.</param>
        /// <param name="message">The result.</param>
        /// <returns>An instance inherited from <see cref="IApiResult{TResult}"/> interface.</returns>
        public static IApiResult<TResult> Succeed<TResult>(TResult result, string message = "请求成功") => new ApiResult<TResult>
        {
            code = 1,
            data = result,
            msg = message
        };

        public static IApiResult Succeed(string message = "请求成功") => new ApiResult
        {
            code = 1,
            msg = message
        };

        /// <summary>
        ///  Creates a new instance of <see cref="IApiResult"/> by the specified error message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="statusCode">The status code</param>
        /// <returns>An instance inherited from <see cref="IApiResult"/> interface.</returns>
        public static IApiResult Failed(string message = "请求失败", int? statusCode = null) => new ApiResult
        {
            code = statusCode ?? 0,
            msg = message
        };

        /// <summary>
        ///  Creates a new instance of <see cref="IApiResult{TResult}"/> by the specified error message.
        /// </summary>
        /// <typeparam name="TResult">The type of the result.</typeparam>
        /// <param name="errorResult">The error result.</param>
        /// <param name="message">The message.</param>
        /// <param name="statusCode">The status code.</param>
        /// <returns>An instance inherited from <see cref="IApiResult"/> interface.</returns>
        public static IApiResult<TResult> Failed<TResult>(TResult errorResult, string message, int? statusCode = null) => new ApiResult<TResult>
        {
            code = statusCode ?? 0,
            msg = message,
            data = errorResult
        };

        /// <summary>
        /// Creates a new instance of <see cref="IApiResult"/> by the specified status code and message.
        /// </summary>
        /// <param name="statusCode">The status code.</param>
        /// <param name="message">The message.</param>
        /// <returns>An instance inherited from <see cref="IApiResult"/> interface.</returns>
        public static IApiResult From(int statusCode, string message = null) => new ApiResult
        {
            code = statusCode,
            msg = message
        };

        /// <summary>
        /// Creates a new instance of <see cref="IApiResult{TResult}"/> by the specified result.
        /// </summary>
        /// <typeparam name="TResult">The type of the result.</typeparam>
        /// <param name="result">The result.</param>
        /// <param name="statusCode">The status code.</param>
        /// <param name="message">The message.</param>
        /// <returns>An instance inherited from <see cref="IApiResult{TResult}"/> interface.</returns>
        public static IApiResult<TResult> From<TResult>(TResult result, int statusCode, string message) => new ApiResult<TResult>
        {
            code = statusCode,
            msg = message,
            data = result
        };
    }
}